/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package chat;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import org.objectweb.dream.message.AbstractChunk;

/**
 * Chunk send to broadcast a chat message.
 */
public class ChatChunk extends AbstractChunk<ChatChunk>
{
  /** default name. */
  public static final String DEFAULT_NAME     = "chat-chunk";

  private static final long  serialVersionUID = 3689351026616514097L;
  private String             message;
  private String             from;

  /**
   * @param from the user name
   * @param message the message.
   */
  public void set(String from, String message)
  {
    this.from = from;
    this.message = message;
  }

  /**
   * @return the user name
   */
  public String getFrom()
  {
    return from;
  }

  /**
   * @return the string message.
   */
  public String getMessage()
  {
    return message;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Chunk interface
  // ---------------------------------------------------------------------------

  /**
   * @see AbstractChunk#newChunk()
   */
  public ChatChunk newChunk()
  {
    return new ChatChunk();
  }

  /**
   * @see AbstractChunk#transfertStateTo(AbstractChunk)
   */
  public void transfertStateTo(ChatChunk newInstance)
  {
    newInstance.set(getFrom(), getMessage());
  }

  /**
   * @see org.objectweb.dream.pool.Recyclable#recycle()
   */
  public void recycle()
  {
    message = null;
    from = null;
  }

  /**
   * @see java.io.Externalizable#readExternal(java.io.ObjectInput)
   */
  public void readExternal(ObjectInput in) throws IOException,
      ClassNotFoundException
  {
    message = in.readUTF();
    from = in.readUTF();
  }

  /**
   * @see java.io.Externalizable#writeExternal(java.io.ObjectOutput)
   */
  public void writeExternal(ObjectOutput out) throws IOException
  {
    out.writeUTF(message);
    out.writeUTF(from);
  }

}
