/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Romain Lenglet
 */

package helloworld;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/** Server component implementation. */
public class ConsumerImpl extends AbstractComponent implements Push
{

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  MessageManager messageManagerItf;

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface
  // ---------------------------------------------------------------------------

  /**
   * @see Push#push(Message)
   */
  public void push(Message message) throws PushException
  {
    HelloWorldChunk chunk = (HelloWorldChunk) messageManagerItf.getChunk(
        message, HelloWorldChunk.DEFAULT_NAME);
    if (chunk == null)
    {
      throw new PushException("Unable to find Helloworld chunk");
    }

    logger.log(BasicLevel.INFO, chunk.getMessage());
    messageManagerItf.deleteMessage(message);
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{MessageManager.ITF_NAME};
  }
}
