/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.channel;

import org.objectweb.fractal.api.control.AttributeController;

/**
 * Attribute controller for {@link DestinationResolver} implementation that use
 * a {@link org.objectweb.dream.protocol.ExportIdentifierChunk} to retrieve
 * destination of messages.
 */
public interface DestinationResolverChunkBasedAttributeController
    extends
      AttributeController
{
  /**
   * Returns the name of the chunks that contains the destination identification
   * of the message.
   * 
   * @return a chunk name.
   */
  String getDestinationChunkName();

  /**
   * Set the name of the chunks that contains the destination identification of
   * the message.
   * 
   * @param destinationChunkName a chunk name.
   */
  void setDestinationChunkName(String destinationChunkName);

  /**
   * Returns <code>true</code> if the destination chunk must be deleted from
   * the message. The default value is <code>true</code>
   * 
   * @return <code>true</code> if the destination chunk must be deleted from
   *         the message.
   */
  boolean getDeleteChunk();

  /**
   * Set if the destination chunk must be deleted from the message.
   * 
   * @param deleteChunk <code>true</code> if the destination chunk must be
   *          deleted from the message.
   */
  void setDeleteChunk(boolean deleteChunk);
}