/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol;

import java.io.IOException;

/**
 * Exception thrown when the binding can't be established.
 */
public class BindException extends IOException
{

  private static final long serialVersionUID = 3258410634088558901L;
  ExportIdentifier          exportId         = null;

  /**
   * @see Exception#Exception()
   */
  public BindException()
  {
  }

  /**
   * @see Exception#Exception(String)
   */
  public BindException(String message)
  {
    super(message);
  }

  /**
   * @param message a message.
   * @param id the identifier of the export on which the binding can't be done.
   */
  public BindException(String message, ExportIdentifier id)
  {
    super(message);
    exportId = id;
  }

  /**
   * Creates an exception caused by another exception.
   * 
   * @param message the detail message.
   * @param id the identifier of the export on which the binding can't be done.
   * @param cause a throwable that cause this exception.
   * @see Exception#Exception(java.lang.String)
   */
  public BindException(String message, ExportIdentifier id, Throwable cause)
  {
    super(message);
    initCause(cause);
    exportId = id;
  }

  /**
   * @see Throwable#getMessage()
   */
  public String getMessage()
  {
    if (exportId == null)
    {
      return super.getMessage();
    }
    return super.getMessage() + " exportId={" + exportId + "}";
  }

}
