/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.bus;

import java.io.IOException;

import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.protocol.IncomingPush;

/**
 * Specialized {@link org.objectweb.dream.protocol.OutgoingPush} interface for
 * bus protocol. Each message is sent to a protocol instance in the bus.
 */
public interface BusOutgoingPush
{

  /**
   * Gives an outgoing message to a protocol.
   * 
   * @param message the message to give to the component.
   * @param to the address in the bus of the receiver of the message.
   * @throws PushException if an exception occurs.
   */
  void outgoingPush(Message message, int to) throws PushException;

  /**
   * Closes the communication channel. The implementation of this method must
   * not call the {@link IncomingPush#incomingClosed} method, since this latter
   * is only used to inform a upper session that a lower session has been closed
   * unexpectedly.
   * 
   * @param incomingPush the interface on which incoming messages of the closed
   *          communication channel were received.
   * @throws IOException the closing process failed.
   */
  void outgoingClose(IncomingPush incomingPush) throws IOException;

  /**
   * Returns the address of this protocol instance in the bus.
   * 
   * @return the address of this protocol instance in the bus.
   */
  int getLocalAddress();

  /**
   * Returns the size of the bus.
   * 
   * @return the size of the bus.
   */
  int getBusSize();
}
