/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.bus.logicalClock;

import java.util.Arrays;

import org.objectweb.dream.message.ChunkFactoryReference;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;

/**
 * TODO Add class comment
 */
public class FIFOClock implements LogicalClock
{
  protected int                                      stamp[];
  protected int                                      lastDelivered[];
  protected int                                      myAddress;
  protected int                                      size                  = -1;
  protected String                                   logicalClockChunkName = LogicalClockChunk.DEFAULT_NAME;
  protected MessageManager                           messageManager;
  protected ChunkFactoryReference<LogicalClockChunk> logicalClockChunkFactory;

  /**
   * @param size the bus size;
   * @param myAddress the local address in the bus.
   * @param logicalClockChunkName the name of the chunk to add in the message.
   * @param messageManager the message manager.
   */
  public FIFOClock(int size, int myAddress, String logicalClockChunkName,
      MessageManager messageManager)

  {
    this.size = size;
    this.myAddress = myAddress;
    this.logicalClockChunkName = logicalClockChunkName;
    this.messageManager = messageManager;
    stamp = new int[size];
    Arrays.fill(stamp, Integer.MIN_VALUE + 1);
    lastDelivered = new int[size];
    Arrays.fill(lastDelivered, Integer.MIN_VALUE);
  }

  // ---------------------------------------------------------------------------
  // Implementation of LogicalClock interface
  // ---------------------------------------------------------------------------

  /**
   * @see LogicalClock#testRecvMessage(Message)
   */
  public synchronized int testRecvMessage(Message message)
  {
    LogicalClockChunk chunk = (LogicalClockChunk) messageManager.getChunk(
        message, logicalClockChunkName);
    int stamp = ((Number) chunk.getLogicalClock()).intValue();
    int from = chunk.getFrom();
    if (stamp <= lastDelivered[from])
    {
      return ALREADY_DELIVERED;
    }
    if (stamp == lastDelivered[from] + 1)
    {
      lastDelivered[from]++;
      messageManager.removeChunk(message, logicalClockChunkName);
      messageManager.deleteChunk(chunk);
      return DELIVER;
    }
    return WAIT_TO_DELIVER;
  }

  /**
   * @see LogicalClock#stamp(Message, int)
   */
  public synchronized void stamp(Message message, int to)
  {
    if (logicalClockChunkFactory == null)
    {
      logicalClockChunkFactory = messageManager
          .getChunkFactory(LogicalClockChunk.class);
    }
    LogicalClockChunk chunk = messageManager
        .createChunk(logicalClockChunkFactory);
    chunk.setFrom(myAddress);
    chunk.setLogicalClock(new Integer(stamp[to]++));
    messageManager.addChunk(message, logicalClockChunkName, chunk);
  }
}
