/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.messagePassing;

import java.util.Map;

import org.objectweb.dream.protocol.ExportException;
import org.objectweb.dream.protocol.IncomingPush;
import org.objectweb.dream.protocol.Protocol;

/**
 * A interface for message passing protocol. The {@link #export} operation
 * creates an access point through which messages can be sent or received. Since
 * this protocol interface provides a connection less protocol abstraction, no
 * bind operation is available.
 */
public interface MessagePassingProtocol extends Protocol
{
  /**
   * If this key is specified in the export hints map, an
   * {@link org.objectweb.dream.protocol.ExportIdentifierChunk} will be added
   * with the given name in every received messages. This chunk will contains
   * the {@link org.objectweb.dream.protocol.ExportIdentifier} of the remote
   * access point that send the message.<br>
   * Depending implementation, some incoming messages may not contain the
   * <i>from chunk</i> if the
   * {@link org.objectweb.dream.protocol.ExportIdentifier} of the remote access
   * point cannot be determined.
   */
  String FROM_CHUNK_NAME = "from-chunk-name";

  /**
   * Creates an access point and returns an interface that can be used to send
   * messages through it. Messages received on this access point will be passed
   * to the given {@link IncomingPush} interface.
   * 
   * @param incomingPushItf the interface on which message received on the
   *          created access point will be passed, may be <code>null</code>.
   * @param hints additional parameters, can be <code>null</code>.
   * @return an interface which can be used to send messages through the created
   *         access point
   * @throws ExportException if the export process fails.
   */
  MessagePassingOutgoingPush export(IncomingPush incomingPushItf,
      Map<String, Object> hints) throws ExportException;
}
