/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.serializator;

import java.io.IOException;

import org.objectweb.dream.message.Message;

/**
 * A serializator defines a method <code>serialize</code> that takes a message
 * as parameter and returns a byte array representation of the message.
 */
public interface Serializator
{

  /** The default name of the {@link Serializator }interface. */
  String ITF_NAME = "serializator";

  /**
   * Serializes the given message and returns a byte array representation of the
   * message.
   * 
   * @param message the message to be serialized.
   * @return a byte array representation of the message.
   * @throws IOException if an exception occurs while serializing the message.
   */
  byte[] serialize(Message message) throws IOException;
}