/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.channel;

import java.util.HashMap;

import junit.framework.TestCase;

import org.objectweb.dream.Push;
import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.dream.test.MessageReceiver;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestTCPChannelInOut extends TestCase
{

  protected Component        testComponent;
  protected Push             outPush1Itf;
  protected MessageReceiver  messageReceiver1Itf;
  protected ExportIdentifier exportId1;

  protected Push             outPush2Itf;
  protected MessageReceiver  messageReceiver2Itf;
  protected ExportIdentifier exportId2;

  protected MessageManager   messageManagerItf;

  protected void setUp() throws Exception
  {
    testComponent = (Component) FactoryFactory.getFactory().newComponent(
        getClass().getName(), new HashMap());
    Fractal.getLifeCycleController(testComponent).startFc();
    outPush1Itf = (Push) testComponent.getFcInterface("in-push1");
    outPush2Itf = (Push) testComponent.getFcInterface("in-push2");

    messageReceiver1Itf = (MessageReceiver) testComponent
        .getFcInterface("message-receiver1");
    messageReceiver2Itf = (MessageReceiver) testComponent
        .getFcInterface("message-receiver2");

    exportId1 = ((ChannelInExportId) testComponent
        .getFcInterface("channel-in-export-id1")).getExportIdentifier();
    exportId2 = ((ChannelInExportId) testComponent
        .getFcInterface("channel-in-export-id2")).getExportIdentifier();

    messageManagerItf = (MessageManager) testComponent
        .getFcInterface(MessageManager.ITF_NAME);
  }

  public void testChannelInOut() throws Exception
  {
    // push message from 1 to 2
    outPush1Itf.push(Util.createMessage(exportId2, messageManagerItf));
    Util.checkMessage(messageReceiver2Itf.waitMessage(), "from2", exportId1,
        messageManagerItf);

    // push message from 2 to 1
    outPush2Itf.push(Util.createMessage(exportId1, messageManagerItf));
    Util.checkMessage(messageReceiver1Itf.waitMessage(), "from1", exportId2,
        messageManagerItf);

    // push message from 1 to 1
    outPush1Itf.push(Util.createMessage(exportId1, messageManagerItf));
    Util.checkMessage(messageReceiver1Itf.waitMessage(), "from1", exportId1,
        messageManagerItf);

    // push message from 2 to 2
    outPush2Itf.push(Util.createMessage(exportId2, messageManagerItf));
    Util.checkMessage(messageReceiver2Itf.waitMessage(), "from2", exportId2,
        messageManagerItf);
  }
}
