/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.bus;

import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.objectweb.dream.IOPushException;
import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.dream.protocol.IPExportIdentifier;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestBus extends TestCase
{
  protected Component      protocolComponent;
  protected BusProtocol    protocol1Itf;
  protected BusProtocol    protocol2Itf;
  protected MessageManager messageManagerItf;

  protected void setUp() throws Exception
  {
    protocolComponent = (Component) FactoryFactory.getFactory().newComponent(
        getClass().getName(), new HashMap());
    Fractal.getLifeCycleController(protocolComponent).startFc();
    protocol1Itf = (BusProtocol) protocolComponent.getFcInterface("protocol1");
    protocol2Itf = (BusProtocol) protocolComponent.getFcInterface("protocol2");
    messageManagerItf = (MessageManager) protocolComponent
        .getFcInterface(MessageManager.ITF_NAME);
  }

  public void testExportPushClose1() throws Exception
  {
    ExportIdentifier id1 = new IPExportIdentifier("localhost", 3000);
    ExportIdentifier id2 = new IPExportIdentifier("localhost", 3001);
    ExportIdentifier busId = protocol1Itf.createExportIdentifier(null,
        new ExportIdentifier[]{id1, id2});

    Map<String, Object> hints1 = new HashMap<String, Object>();
    hints1.put(BusProtocol.LOCAL_ADDRESS_HINT_KEY, new Integer(0));
    Session s1 = new Session();
    s1.outgoingPush = protocol1Itf.bind(busId, s1, hints1);

    Map<String, Object> hints2 = new HashMap<String, Object>();
    hints2.put(BusProtocol.LOCAL_ADDRESS_HINT_KEY, new Integer(1));
    Session s2 = new Session();
    s2.outgoingPush = protocol1Itf.bind(busId, s2, hints2);

    // push message from 1 to 2
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(), 1);
    s2.waitMessage();

    // push message from 2 to 1
    s2.outgoingPush.outgoingPush(messageManagerItf.createMessage(), 0);
    s1.waitMessage();

    // close s1
    s1.outgoingPush.outgoingClose(s1);
    // try to push on closed 1
    try
    {
      s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(), 1);
      fail("outgoingPush should throw an exception here");
    }
    catch (IOPushException e)
    {
      // ok
    }

    // push message from 2 to 2
    s2.outgoingPush.outgoingPush(messageManagerItf.createMessage(), 1);
    s2.waitMessage();

    // close s2
    s2.outgoingPush.outgoingClose(s2);
  }
}
