/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.channel;

import junit.framework.TestCase;

import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.protocol.IncomingPush;
import org.objectweb.dream.protocol.OutgoingPush;

class Session implements IncomingPush
{
  OutgoingPush    outgoingPush;
  private Message serverReceivedMessage;
  private boolean serverSessionClosed;

  Session()
  {
  }

  Session(OutgoingPush push)
  {
    outgoingPush = push;
  }

  protected synchronized Message waitMessage() throws InterruptedException
  {
    Message m;
    if (serverReceivedMessage == null)
    {
      wait(1000);
    }
    TestCase.assertNotNull(serverReceivedMessage);
    m = serverReceivedMessage;
    serverReceivedMessage = null;
    return m;
  }

  protected synchronized void waitClosed() throws InterruptedException
  {
    if (!serverSessionClosed)
    {
      wait(1000);
    }
    TestCase.assertTrue(serverSessionClosed);
    serverSessionClosed = false;
  }

  public synchronized void incomingPush(Message message) throws PushException
  {
    serverReceivedMessage = message;
    notifyAll();
  }

  public synchronized void incomingClosed(Object outgoingPush,
      Exception exception)
  {
    TestCase.assertEquals("Given outgoingPush is not correct",
        this.outgoingPush, outgoingPush);
    serverSessionClosed = true;
    notifyAll();
  }
}