/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.messagePassing;

import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.objectweb.dream.IOPushException;
import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.dream.message.AbstractChunk;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.dream.protocol.ExportIdentifierChunk;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestMessagePassingOverChannel1 extends TestCase
{
  protected Component              protocolComponent;
  protected MessagePassingProtocol protocol1Itf;
  protected MessagePassingProtocol protocol2Itf;
  protected MessagePassingProtocol protocol3Itf;
  protected MessageManager         messageManagerItf;

  protected void setUp() throws Exception
  {
    protocolComponent = (Component) FactoryFactory.getFactory().newComponent(
        getClass().getName(), new HashMap());
    Fractal.getLifeCycleController(protocolComponent).startFc();
    protocol1Itf = (MessagePassingProtocol) protocolComponent
        .getFcInterface("protocol1");
    protocol2Itf = (MessagePassingProtocol) protocolComponent
        .getFcInterface("protocol2");
    protocol3Itf = (MessagePassingProtocol) protocolComponent
        .getFcInterface("protocol3");
    messageManagerItf = (MessageManager) protocolComponent
        .getFcInterface(MessageManager.ITF_NAME);
  }

  public void testExportPushClose1() throws Exception
  {
    Session s1 = new Session();
    s1.outgoingPush = protocol1Itf.export(s1, null);

    Session s2 = new Session();
    s2.outgoingPush = protocol2Itf.export(s2, null);

    // push message from 1 to 2
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s2.outgoingPush.getLocalExportIdentifier());
    s2.waitMessage();

    // push message from 2 to 1
    s2.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s1.outgoingPush.getLocalExportIdentifier());
    s1.waitMessage();

    // close s2
    s2.outgoingPush.outgoingClose(s2);
    Thread.sleep(100);

    // try to push from 1 to 2
    try
    {
      s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
          s2.outgoingPush.getLocalExportIdentifier());
      fail("outgoingPush should throw an exception here");
    }
    catch (IOPushException e1)
    {
      // ok
    }

    // close s1
    s1.outgoingPush.outgoingClose(s1);
    // try to push on closed 1
    try
    {
      s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
          s2.outgoingPush.getLocalExportIdentifier());
      fail("outgoingPush should throw an exception here");
    }
    catch (IOPushException e)
    {
      // ok
    }
    return;
  }

  public void testExportPushClose2() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(MessagePassingProtocol.FROM_CHUNK_NAME, "from");
    Session s1 = new Session();
    s1.outgoingPush = protocol1Itf.export(s1, hints);

    Session s2 = new Session();
    s2.outgoingPush = protocol2Itf.export(s2, hints);

    Session s3 = new Session();
    s3.outgoingPush = protocol3Itf.export(s3, hints);

    // push message from 1 to 2
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s2.outgoingPush.getLocalExportIdentifier());
    checkFrom(s2.waitMessage(), s1.outgoingPush.getLocalExportIdentifier());

    // push message from 1 to 3
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s3.outgoingPush.getLocalExportIdentifier());
    checkFrom(s3.waitMessage(), s1.outgoingPush.getLocalExportIdentifier());

    // push message from 2 to 3
    s2.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s3.outgoingPush.getLocalExportIdentifier());
    checkFrom(s3.waitMessage(), s2.outgoingPush.getLocalExportIdentifier());

    // push message from 1 to 2
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s2.outgoingPush.getLocalExportIdentifier());
    checkFrom(s2.waitMessage(), s1.outgoingPush.getLocalExportIdentifier());

    // push message from 2 to 1
    s2.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s1.outgoingPush.getLocalExportIdentifier());
    checkFrom(s1.waitMessage(), s2.outgoingPush.getLocalExportIdentifier());

    // push message from 3 to 1
    s3.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s1.outgoingPush.getLocalExportIdentifier());
    checkFrom(s1.waitMessage(), s3.outgoingPush.getLocalExportIdentifier());

    // push message from 3 to 2
    s3.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s2.outgoingPush.getLocalExportIdentifier());
    checkFrom(s2.waitMessage(), s3.outgoingPush.getLocalExportIdentifier());

    // push message from 1 to 1
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s1.outgoingPush.getLocalExportIdentifier());
    checkFrom(s1.waitMessage(), s1.outgoingPush.getLocalExportIdentifier());

    // push message from 2 to 2
    s2.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s2.outgoingPush.getLocalExportIdentifier());
    checkFrom(s2.waitMessage(), s2.outgoingPush.getLocalExportIdentifier());

    // push message from 3 to 3
    s3.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s3.outgoingPush.getLocalExportIdentifier());
    checkFrom(s3.waitMessage(), s3.outgoingPush.getLocalExportIdentifier());

    // close s2
    s2.outgoingPush.outgoingClose(s2);
    Thread.sleep(100);

    // try to push from 1 to 2
    try
    {
      s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
          s2.outgoingPush.getLocalExportIdentifier());
      fail("outgoingPush should throw an exception here");
    }
    catch (IOPushException e1)
    {
      // ok
    }

    // close s1
    s1.outgoingPush.outgoingClose(s1);
    // try to push on closed 1
    try
    {
      s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
          s2.outgoingPush.getLocalExportIdentifier());
      fail("outgoingPush should throw an exception here");
    }
    catch (IOPushException e)
    {
      // ok
    }

    // close s3
    s3.outgoingPush.outgoingClose(s3);
    // try to push on closed 3
    try
    {
      s3.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
          s1.outgoingPush.getLocalExportIdentifier());
      fail("outgoingPush should throw an exception here");
    }
    catch (IOPushException e)
    {
      // ok
    }
  }

  private void checkFrom(Message message, ExportIdentifier id)
  {
    AbstractChunk chunk = messageManagerItf.getChunk(message, "from");
    assertNotNull(chunk);
    assertTrue(chunk instanceof ExportIdentifierChunk);
    ExportIdentifier from = ((ExportIdentifierChunk) chunk)
        .getExportIdentifier();
    assertEquals(id, from);
  }
}
