/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.messagePassing;

import java.util.HashMap;

import junit.framework.TestCase;

import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestMessagePassingOverChannel2 extends TestCase
{
  protected Component              protocolComponent;
  protected MessagePassingProtocol protocol1Itf;
  protected MessagePassingProtocol protocol2Itf;
  protected MessageManager         messageManagerItf;

  protected class Sender implements Runnable
  {

    MessagePassingOutgoingPush outgoingPush;
    ExportIdentifier           to;

    Sender(MessagePassingOutgoingPush push, ExportIdentifier to)
    {
      outgoingPush = push;
      this.to = to;
    }

    public void run()
    {
      try
      {
        outgoingPush.outgoingPush(messageManagerItf.createMessage(), to);
      }
      catch (Exception e)
      {
        e.printStackTrace();
        fail("Exception caught while pushing message :" + e);
      }
    }

  }

  protected void setUp() throws Exception
  {
    protocolComponent = (Component) FactoryFactory.getFactory().newComponent(
        getClass().getName(), new HashMap());
    Fractal.getLifeCycleController(protocolComponent).startFc();
    protocol1Itf = (MessagePassingProtocol) protocolComponent
        .getFcInterface("protocol1");
    protocol2Itf = (MessagePassingProtocol) protocolComponent
        .getFcInterface("protocol2");
    messageManagerItf = (MessageManager) protocolComponent
        .getFcInterface(MessageManager.ITF_NAME);
  }

  public void testExportSimultaneousPush() throws Exception
  {
    Session s1 = new Session();
    s1.outgoingPush = protocol1Itf.export(s1, null);

    Session s2 = new Session();
    s2.outgoingPush = protocol2Itf.export(s2, null);

    Thread sender1 = new Thread(new Sender(s1.outgoingPush, s2.outgoingPush
        .getLocalExportIdentifier()));
    Thread sender2 = new Thread(new Sender(s2.outgoingPush, s1.outgoingPush
        .getLocalExportIdentifier()));

    sender1.start();
    sender2.start();

    // Thread.sleep(200);
    s1.waitMessage();
    s2.waitMessage();

    s1.outgoingPush.outgoingClose(s1);
    Thread.sleep(200);
    s2.outgoingPush.outgoingClose(s2);
  }
}
