/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.messagePassing;

import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.objectweb.dream.IOPushException;
import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.protocol.channel.TCPIPProtocol;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestReliableMessagePassing extends TestCase
{
  protected Component              protocolComponent;
  protected MessagePassingProtocol protocol1Itf;
  protected MessagePassingProtocol protocol2Itf;
  protected MessagePassingProtocol protocol3Itf;
  protected MessageManager         messageManagerItf;

  protected void setUp() throws Exception
  {
    protocolComponent = (Component) FactoryFactory.getFactory().newComponent(
        getClass().getName(), new HashMap());
    Fractal.getLifeCycleController(protocolComponent).startFc();
    protocol1Itf = (MessagePassingProtocol) protocolComponent
        .getFcInterface("protocol1");
    protocol2Itf = (MessagePassingProtocol) protocolComponent
        .getFcInterface("protocol2");
    protocol3Itf = (MessagePassingProtocol) protocolComponent
        .getFcInterface("protocol3");
    messageManagerItf = (MessageManager) protocolComponent
        .getFcInterface(MessageManager.ITF_NAME);
  }

  public void testExportPushClose1() throws Exception
  {
    Map<String, Object> hints1 = new HashMap<String, Object>();
    hints1.put(ReliableMessagePassingProtocol.RETRANSMIT_PERIOD_HINT_KEY,
        new Long(500));
    hints1.put(TCPIPProtocol.PORT, new Integer(3000));
    hints1.put(TCPIPProtocol.PORT_RANGE, new Integer(0));
    Session s1 = new Session();
    s1.outgoingPush = protocol1Itf.export(s1, hints1);

    Map<String, Object> hints2 = new HashMap<String, Object>();
    hints2.put(ReliableMessagePassingProtocol.RETRANSMIT_PERIOD_HINT_KEY,
        new Long(500));
    hints2.put(TCPIPProtocol.PORT, new Integer(3001));
    hints2.put(TCPIPProtocol.PORT_RANGE, new Integer(0));
    Session s2 = new Session();
    s2.outgoingPush = protocol2Itf.export(s2, hints2);

    Map<String, Object> hints3 = new HashMap<String, Object>();
    hints3.put(ReliableMessagePassingProtocol.RETRANSMIT_PERIOD_HINT_KEY,
        new Long(500));
    hints3.put(TCPIPProtocol.PORT, new Integer(3002));
    hints3.put(TCPIPProtocol.PORT_RANGE, new Integer(0));
    Session s3 = new Session();
    s3.outgoingPush = protocol3Itf.export(s3, hints3);

    // push message from 1 to 2
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s2.outgoingPush.getLocalExportIdentifier());
    s2.waitMessage();

    // push message from 1 to 3
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s3.outgoingPush.getLocalExportIdentifier());
    s3.waitMessage();

    // push message from 2 to 1
    s2.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s1.outgoingPush.getLocalExportIdentifier());
    s1.waitMessage();

    // push message from 2 to 3
    s2.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s3.outgoingPush.getLocalExportIdentifier());
    s3.waitMessage();

    // close s2
    s2.outgoingPush.outgoingClose(s2);
    Thread.sleep(100);

    // try to push from 1 to 2
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s2.outgoingPush.getLocalExportIdentifier());

    // push message from 1 to 3
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s3.outgoingPush.getLocalExportIdentifier());
    s3.waitMessage();

    // push message from 3 to 1
    s3.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s1.outgoingPush.getLocalExportIdentifier());
    s1.waitMessage();

    // reopen s2
    s2.outgoingPush = protocol2Itf.export(s2, hints2);
    s2.waitMessage();

    // close s1
    s1.outgoingPush.outgoingClose(s1);
    // try to push on closed 1
    try
    {
      s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
          s2.outgoingPush.getLocalExportIdentifier());
      fail("outgoingPush should throw an exception here");
    }
    catch (IOPushException e)
    {
      // ok
    }

    // close s2
    s2.outgoingPush.outgoingClose(s2);

    // close s3
    s3.outgoingPush.outgoingClose(s3);
  }

}
