/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.messagePassing;

import java.io.IOException;
import java.util.HashMap;

import junit.framework.TestCase;

import org.objectweb.dream.IOPushException;
import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.dream.message.AbstractChunk;
import org.objectweb.dream.message.ChunkFactoryReference;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.protocol.ExceptionChunk;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestSerializatorMessagePassing extends TestCase
{
  protected Component                             protocolComponent;
  protected MessagePassingProtocol                protocol1Itf;
  protected MessagePassingProtocol                protocol2Itf;
  protected MessageManager                        messageManagerItf;
  protected ChunkFactoryReference<ExceptionChunk> exceptionChunkFactoryReference;

  protected void setUp() throws Exception
  {
    protocolComponent = (Component) FactoryFactory.getFactory().newComponent(
        getClass().getName(), new HashMap());
    Fractal.getLifeCycleController(protocolComponent).startFc();
    protocol1Itf = (MessagePassingProtocol) protocolComponent
        .getFcInterface("protocol1");
    protocol2Itf = (MessagePassingProtocol) protocolComponent
        .getFcInterface("protocol2");
    messageManagerItf = (MessageManager) protocolComponent
        .getFcInterface(MessageManager.ITF_NAME);
    exceptionChunkFactoryReference = messageManagerItf
        .getChunkFactory(ExceptionChunk.class);
  }

  public void testExportPushClose1() throws Exception
  {
    Session s1 = new Session();
    s1.outgoingPush = protocol1Itf.export(s1, null);

    Session s2 = new Session();
    s2.outgoingPush = protocol2Itf.export(s2, null);

    // push message from 1 to 2
    Message message = messageManagerItf.createMessage();
    // use exception chunk as content of message.
    Exception testException = new IOException("just for test");
    ExceptionChunk exceptionChunk = messageManagerItf
        .createChunk(exceptionChunkFactoryReference);
    exceptionChunk.setException(testException);
    messageManagerItf.addChunk(message, "toto", exceptionChunk);
    s1.outgoingPush.outgoingPush(message, s2.outgoingPush
        .getLocalExportIdentifier());
    Message receivedMessage = s2.waitMessage();
    checkMessage(testException, receivedMessage);

    // push message from 2 to 1
    message = messageManagerItf.createMessage();
    // use exception chunk as content of message.
    testException = new IOException("just a reply");
    exceptionChunk = messageManagerItf
        .createChunk(exceptionChunkFactoryReference);
    exceptionChunk.setException(testException);
    messageManagerItf.addChunk(message, "toto", exceptionChunk);
    s2.outgoingPush.outgoingPush(message, s1.outgoingPush
        .getLocalExportIdentifier());
    checkMessage(testException, s1.waitMessage());

    // push message from 1 to 1
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s1.outgoingPush.getLocalExportIdentifier());
    s1.waitMessage();

    // push message from 2 to 2
    s2.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s2.outgoingPush.getLocalExportIdentifier());
    s2.waitMessage();

    // close s1
    s1.outgoingPush.outgoingClose(s1);
    // try to push on closed 1
    try
    {
      s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
          s2.outgoingPush.getLocalExportIdentifier());
      fail("outgoingPush should throw an exception here");
    }
    catch (IOPushException e)
    {
      // ok
    }

    // close s2
    s2.outgoingPush.outgoingClose(s2);
  }

  private void checkMessage(Exception testException, Message receivedMessage)
  {
    AbstractChunk chunk = messageManagerItf.getChunk(receivedMessage, "toto");
    assertNotNull(chunk);
    assertTrue(chunk instanceof ExceptionChunk);
    assertNotNull(((ExceptionChunk) chunk).getException());
    assertEquals(testException.getMessage(), ((ExceptionChunk) chunk)
        .getException().getMessage());
  }

}
