/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.queue;

import java.util.HashMap;

import junit.framework.TestCase;

import org.objectweb.dream.Pull;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestPushPullFIFO1 extends TestCase
{

  protected Component      testComponent;
  protected Push           pushItf;
  protected Pull           pullItf;

  protected MessageManager messageManagerItf;

  protected void setUp() throws Exception
  {
    testComponent = (Component) FactoryFactory.getFactory().newComponent(
        getClass().getName(), new HashMap());
    Fractal.getLifeCycleController(testComponent).startFc();
    pushItf = (Push) testComponent.getFcInterface("push");
    pullItf = (Pull) testComponent.getFcInterface("pull");

    messageManagerItf = (MessageManager) testComponent
        .getFcInterface(MessageManager.ITF_NAME);
  }

  public void testPushPull() throws Exception
  {
    pushItf.push(Util.createMessage(0, messageManagerItf));
    Util.checkMessage(pullItf.pull(), 0, messageManagerItf);
    assertNull(pullItf.pull());

    pushItf.push(Util.createMessage(1, messageManagerItf));
    pushItf.push(Util.createMessage(2, messageManagerItf));
    Util.checkMessage(pullItf.pull(), 1, messageManagerItf);
    Util.checkMessage(pullItf.pull(), 2, messageManagerItf);

    pushItf.push(Util.createMessage(5, messageManagerItf));
    pushItf.push(Util.createMessage(6, messageManagerItf));
    pushItf.push(Util.createMessage(4, messageManagerItf));
    pushItf.push(Util.createMessage(3, messageManagerItf));
    Util.checkMessage(pullItf.pull(), 5, messageManagerItf);
    Util.checkMessage(pullItf.pull(), 6, messageManagerItf);
    Util.checkMessage(pullItf.pull(), 4, messageManagerItf);
    Util.checkMessage(pullItf.pull(), 3, messageManagerItf);

    pushItf.push(Util.createMessage(7, messageManagerItf));
    Util.checkMessage(pullItf.pull(), 7, messageManagerItf);
  }

  public void testNonBlockingHandler() throws Exception
  {
    pushItf.push(Util.createMessage(0, messageManagerItf));
    Util.checkMessage(pullItf.pull(), 0, messageManagerItf);

    pushItf.push(Util.createMessage(12, messageManagerItf));
    pushItf.push(Util.createMessage(11, messageManagerItf));
    pushItf.push(Util.createMessage(10, messageManagerItf));
    pushItf.push(Util.createMessage(9, messageManagerItf));
    pushItf.push(Util.createMessage(8, messageManagerItf));
    pushItf.push(Util.createMessage(7, messageManagerItf));
    pushItf.push(Util.createMessage(6, messageManagerItf));
    pushItf.push(Util.createMessage(5, messageManagerItf));
    pushItf.push(Util.createMessage(4, messageManagerItf));
    pushItf.push(Util.createMessage(3, messageManagerItf));
    try
    {
      pushItf.push(Util.createMessage(2, messageManagerItf));
      fail("push should throw an exception here");
    }
    catch (PushException e)
    {
      assertTrue(e instanceof BufferOverflowException);
    }
  }
}
