/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.queue;

import java.util.ArrayList;
import java.util.Iterator;

import junit.framework.TestCase;

import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;

final class Util
{
  static Message createMessage(long seqno, MessageManager messageManagerItf)
  {
    Message message = messageManagerItf.createMessage();

    SequenceNumberChunk chunk = messageManagerItf.createChunk(messageManagerItf
        .getChunkFactory(SequenceNumberChunk.class));
    chunk.setSequenceNumber(seqno);
    messageManagerItf.addChunk(message, "seqno", chunk);
    return message;
  }

  static void checkMessage(Message message, long seqno,
      MessageManager messageManagerItf)
  {
    TestCase.assertNotNull(message);
    SequenceNumberChunk chunk = (SequenceNumberChunk) messageManagerItf
        .getChunk(message, "seqno");
    TestCase.assertEquals(seqno, chunk.getSequenceNumber());
  }

  static Message[] chekAggregateMessage(Message message, int nbSubMessage,
      MessageManager messageManagerItf)
  {
    TestCase.assertNotNull(message);
    ArrayList<Message> msgs = new ArrayList<Message>();
    Iterator<Message> iter = messageManagerItf.getSubMessageIterator(message);
    while (iter.hasNext())
    {
      Message submsg = iter.next();
      msgs.add(submsg);
    }
    TestCase.assertEquals(nbSubMessage, msgs.size());
    return msgs.toArray(new Message[msgs.size()]);
  }

}
