/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.test;

import junit.framework.TestCase;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.util.EmptyStringArray;

public class MessageReceiverImpl extends AbstractComponent
    implements
      Push,
      MessageReceiver
{

  Message receivedMessage;

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface
  // ---------------------------------------------------------------------------

  /**
   * @see Push#push(Message)
   */
  public synchronized void push(Message message) throws PushException
  {
    receivedMessage = message;
    notify();
  }

  // ---------------------------------------------------------------------------
  // Implementation of the MessageReceiver interface
  // ---------------------------------------------------------------------------

  /**
   * @see MessageReceiver#waitMessage()
   */
  public synchronized Message waitMessage() throws InterruptedException
  {
    if (receivedMessage == null)
    {
      wait(1000);
    }
    TestCase.assertNotNull(receivedMessage);
    Message m = receivedMessage;
    receivedMessage = null;
    return m;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  public String[] listFc()
  {
    return EmptyStringArray.EMPTY_STRING_ARRAY;
  }

}
