/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema, Romain Lenglet
 */

package org.objectweb.dream;

import org.objectweb.dream.message.Message;

/**
 * Common <I>Pull </I> interface for Dream components. This interface defines a
 * method {@link #pull() <code>pull</code>} that allows a component to ask
 * another component for a message.
 */
public interface Pull
{

  /**
   * The commonly used name to refer to this interface when it is used as an
   * output.
   */
  String OUT_PULL_ITF_NAME = "out-pull";

  /**
   * The commonly used name to refer to this interface when it is used as an
   * input.
   */
  String IN_PULL_ITF_NAME  = "in-pull";

  /**
   * Asks for a message. If no message is available, the semantic of this method
   * is left to the component implementing the interface. As an example, it can
   * return <code>null</code> or block until a message is available.
   * 
   * @return the pulled message, or <code>null</code>.
   * @throws PullException if an exception occurs.
   */
  Message pull() throws PullException;
}