/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.adl;

import java.util.Map;

import org.objectweb.dream.adl.legacy.ComponentNotFoundException;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.util.Fractal;

/**
 * Resolver component to find sub component in a composite.
 */
public class FractalResolver implements Resolver
{

  /**
   * The <code>namingContext</code> object is used as a composite component.
   * The <code>name</code> paramaeter is a name of a sub component. <br>
   * This method returns a sub component of the given composite with the
   * specified name.
   * 
   * @see Resolver#resolve(Object, String, Map)
   */
  public Object resolve(Object namingContext, String name, Map context)
      throws ComponentNotFoundException
  {
    Component composite = (Component) namingContext;
    ContentController cc;
    try
    {
      cc = Fractal.getContentController(composite);
    }
    catch (NoSuchInterfaceException e)
    {
      throw new ComponentNotFoundException(
          "The given component doesn't have a Content Controller", context,
          name);
    }
    Component subComponents[] = cc.getFcSubComponents();
    for (int i = 0; i < subComponents.length; i++)
    {
      Component subComponent = subComponents[i];
      NameController nc;
      try
      {
        nc = Fractal.getNameController(subComponent);
      }
      catch (NoSuchInterfaceException e1)
      {
        break;
      }
      if (nc.getFcName().equals(name))
      {
        return subComponent;
      }
    }
    throw new ComponentNotFoundException(
        "Unable to find sub component with the given name", context, name);
  }

}