/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.activity.manager;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.control.activity.task.IllegalTaskException;
import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.dream.control.activity.task.thread.ThreadTaskImpl;
import org.objectweb.dream.control.content.BasicContentController;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Implementation of the {@link ThreadManager }interface.
 */
public class ThreadManagerImpl extends AbstractComponent
    implements
      ThreadManager
{

  // ---------------------------------------------------------------------------
  // client interfaces
  // ---------------------------------------------------------------------------

  /** Name of the content controller client interface of this component. */
  public static final String ACTIVITY_CONTENT_CONTROLLER_ITF_NAME = "activities-content";
  BasicContentController     contentControllerItf;

  // ---------------------------------------------------------------------------
  // Implementation of the ThreadManager interface
  // ---------------------------------------------------------------------------

  /**
   * @see ThreadManager#addThread()
   */
  public Task addThread()
  {
    Task thread = new ThreadTaskImpl();
    try
    {
      ((BindingController) thread).bindFc("logger", logger);
      contentControllerItf.addFcSubComponent(thread);
    }
    catch (Exception e)
    {
      Error.bug(logger, e);
    }
    return thread;
  }

  /**
   * @see ThreadManager#removeThread(Task)
   */
  public void removeThread(Task task) throws IllegalTaskException
  {
    try
    {
      contentControllerItf.removeFcSubComponent(task);
    }
    catch (IllegalContentException e)
    {
      throw new IllegalTaskException(task, "unknown task", e);
    }
    catch (IllegalLifeCycleException e)
    {
      Error.bug(logger, e);
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{ACTIVITY_CONTENT_CONTROLLER_ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(ACTIVITY_CONTENT_CONTROLLER_ITF_NAME))
    {
      contentControllerItf = (BasicContentController) serverItf;
    }
  }

}
