/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.activity.manager;

import org.objectweb.dream.control.activity.task.IllegalTaskException;
import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.dream.control.activity.task.thread.ThreadPoolOverflowException;

/**
 * Interface used to control the multi threaded execution of a task.
 */
public interface ThreadPoolManager
{
  /** Defualt name of this interface. */
  String ITF_NAME = "thread-pool-manager";

  /**
   * Returns the number of currently active threads for the given task.
   * 
   * @param task a task;
   * @return the number of currently active threads for the given task.
   * @throws IllegalTaskException if the given task is unknown
   */
  int getNbActiveThreads(Task task) throws IllegalTaskException;

  /**
   * Adds a thread to the given task.
   * 
   * @param task a task
   * @throws ThreadPoolOverflowException if the thread can't be allocated (pool
   *           full for instance).
   * @throws IllegalTaskException if the given task is unknown
   */
  void addThread(Task task) throws ThreadPoolOverflowException,
      IllegalTaskException;
}
