/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.control.activity.task.thread;

import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.julia.control.lifecycle.ChainedIllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Basic implementation of a Dream thread.
 * 
 * @see AbstractThreadTaskImpl
 */
public class ThreadTaskImpl extends AbstractThreadTaskImpl implements Runnable
{
  protected Thread  thread = null;

  protected boolean executing;

  // ---------------------------------------------------------------------------
  // Implementation of the AbstractThreadTask abstract methods
  // ---------------------------------------------------------------------------

  /**
   * @see AbstractThreadTaskImpl#isExecuting()
   */
  protected boolean isExecuting()
  {
    return executing;
  }

  /**
   * @see AbstractThreadTaskImpl#setExecuting(boolean)
   */
  protected void setExecuting(boolean b)
  {
    executing = b;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Runnable interface
  // ---------------------------------------------------------------------------

  /**
   * @see java.lang.Runnable#run()
   */
  public void run()
  {
    logger.log(BasicLevel.DEBUG, "Begin of the run method");
    try
    {
      execute(null);
    }
    catch (InterruptedException e)
    {
      logger.log(BasicLevel.DEBUG, "Interrupted");
    }
    finally
    {
      try
      {
        super.stopFc();
      }
      catch (IllegalLifeCycleException ignored)
      {
        // throws nothing
      }
      thread = null;
      logger.log(BasicLevel.DEBUG, "End of the run method");
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the LifeCycleController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    if (getFcState() == STARTED)
    {
      return;
    }

    // sanity check.
    if (thread != null)
    {
      Error.bug(logger, new IllegalStateException(
          "Thread component is not in the start state, but has a thread"));
    }

    super.startFc();
    executing = true;
    thread = new Thread(this);
    logger.log(BasicLevel.DEBUG, "Starting the thread task");
    thread.start();
  }

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
   */
  public void stopFc() throws IllegalLifeCycleException
  {
    Thread t;
    if (getFcState() == STOPPED)
    {
      return;
    }

    // sanity check.
    if (thread == null)
    {
      Error.bug(logger, new IllegalStateException(
          "Thread component is in start state, but has no thread"));
    }

    executing = false;
    logger.log(BasicLevel.DEBUG, "Interrupting the thread task");
    thread.interrupt();
    t = thread;
    try
    {
      logger.log(BasicLevel.DEBUG, "Join thread task");
      t.join();
    }
    catch (InterruptedException e)
    {
      throw new ChainedIllegalLifeCycleException(e, null,
          "Interrupted while waiting for the end of the thread.");
    }
    logger.log(BasicLevel.DEBUG, "Thread task stop");
  }
}