/**
 * Dream Copyright (C) 2003-2004 INRIA Rhone-Alpes
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Contact: dream@objectweb.org
 * 
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.control.lifecycle;

import org.objectweb.dream.control.logger.LoggerController;
import org.objectweb.dream.control.logger.LoggerInternalBaseName;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.julia.Util;
import org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator;
import org.objectweb.fractal.util.Fractal;

/**
 * Provides an implementation of the {@link LifeCycleController}interface. This
 * implementation simply registers <br>
 * <br>
 * <b>Requirements </b>
 * <ul>
 * <li>this mixin requires the
 * {@link org.objectweb.fractal.julia.UseComponentMixin }mixin.</li>
 * <li>this mixin requires the
 * {@link org.objectweb.dream.control.logger.UseLoggerControllerMixin }mixin.
 * </li>
 * <li>the component to which this controller belongs should provide the
 * {@link org.objectweb.dream.control.logger.LoggerInternalBaseName }interface.
 * </li>
 * </ul>
 */

public abstract class LoggerLifeCycleControllerMixin
    implements
      LifeCycleCoordinator,
      LifeCycleController
{
  String fcState = STOPPED;

  // -------------------------------------------------------------------------
  // Private constructor
  // -------------------------------------------------------------------------

  private LoggerLifeCycleControllerMixin()
  {
  }

  // -------------------------------------------------------------------------
  // Implementation of the LifeCycleController interface
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
   */
  public String getFcState()
  {
    return fcState;
  }

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    fcState = STARTED;
    fcState = STARTED;
    StringBuffer path = new StringBuffer();
    Util.toString(_this_weaveableC, path);
    String s = path.toString().substring(1).replace('/', '.');
    if (_this_weaveableOptLogC instanceof LoggerInternalBaseName)
    {
      ((LoggerInternalBaseName) _this_weaveableOptLogC).internalSetBaseName(s);
    }
    else
    {
      _this_weaveableOptLogC.setBaseName(s);
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
   */
  public void stopFc() throws IllegalLifeCycleException
  {
    fcState = STOPPED;
  }

  // -------------------------------------------------------------------------
  // Implementation of the LifeCycleCoordinator interface
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator#fcActivated(org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator)
   */
  public boolean fcActivated(LifeCycleCoordinator component)
  {
    return true;
  }

  /**
   * @see org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator#fcInactivated(org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator)
   */
  public void fcInactivated(LifeCycleCoordinator component)
  {

  }

  /**
   * @see org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator#setFcStarted()
   */
  public boolean setFcStarted() throws IllegalLifeCycleException
  {
    fcState = STARTED;
    StringBuffer path = new StringBuffer();
    Util.toString(_this_weaveableC, path);
    String s = path.toString().substring(1).replace('/', '.');
    if (_this_weaveableOptLogC instanceof LoggerInternalBaseName)
    {
      ((LoggerInternalBaseName) _this_weaveableOptLogC).internalSetBaseName(s);
    }
    else
    {
      _this_weaveableOptLogC.setBaseName(s);
    }
    return true;
  }

  /**
   * @see org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator#setFcStopped()
   */
  public boolean setFcStopped() throws IllegalLifeCycleException
  {
    fcState = STOPPED;
    return true;
  }

  /**
   * @see org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator#setFcStopping(org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator)
   */
  public void setFcStopping(LifeCycleCoordinator coordinator)
      throws IllegalLifeCycleException
  {
    try
    {
      coordinator.fcInactivated((LifeCycleCoordinator) Fractal
          .getLifeCycleController(_this_weaveableC));
    }
    catch (NoSuchInterfaceException e)
    {
      Error.bug(null, e);
    }
  }

  // -------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // -------------------------------------------------------------------------

  /**
   * The <tt>weaveableC</tt> field required by this mixin. This field is
   * supposed to reference the {@link Component }interface of the component to
   * which this controller object belongs.
   */
  public Component        _this_weaveableC;

  /**
   * The <tt>weaveableOptLogC</tt> field required by this mixin. This field is
   * supposed to reference the {@link LoggerController }interface of the
   * component to which this controller object belongs.
   */
  public LoggerController _this_weaveableOptLogC;

}