/**
 * Dream Copyright (C) 2003-2004 INRIA Rhone-Alpes
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Contact: dream@objectweb.org
 * 
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.control.location;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;

/**
 * Basic Implementation of the {@link LocationController}.
 */
public abstract class BasicLocationControllerMixin
    implements
      LocationController
{
  protected Component myBootstrap;

  // -------------------------------------------------------------------------
  // Private constructor
  // -------------------------------------------------------------------------

  private BasicLocationControllerMixin()
  {
  }

  // ---------------------------------------------------------------------------
  // Implementation of the LocationController interface
  // ---------------------------------------------------------------------------

  /**
   * @see LocationController#getBootstraps()
   */
  public Component[] getBootstraps()
  {
    if (_this_weaveableOptCC == null)
    {
      return new Component[]{myBootstrap};
    }
    Set<Component> bootstraps = new HashSet<Component>();
    Component[] subComponents = _this_weaveableOptCC.getFcSubComponents();
    for (int i = 0; i < subComponents.length; i++)
    {
      Component component = subComponents[i];
      LocationController locationController;
      try
      {
        locationController = (LocationController) component
            .getFcInterface("location-controller");
      }
      catch (NoSuchInterfaceException e)
      {
        continue;
      }
      Component[] subBootstraps = locationController.getBootstraps();
      for (int j = 0; j < subBootstraps.length; j++)
      {
        bootstraps.add(subBootstraps[j]);
      }
    }
    bootstraps.remove(myBootstrap);
    Component[] tab = new Component[bootstraps.size() + 1];
    tab[0] = myBootstrap;
    Iterator<Component> iterator = bootstraps.iterator();
    int i = 1;
    while (iterator.hasNext())
    {
      tab[i++] = iterator.next();
    }
    return tab;
  }

  /**
   * @see LocationController#setCreationBootstrap(Component)
   */
  public void setCreationBootstrap(Component bootstrap)
  {
    myBootstrap = bootstrap;
  }

  // --------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // --------------------------------------------------------------------------

  /**
   * The <tt>weaveableOptCC</tt> field required by this mixin. This field is
   * supposed to reference the {@link ContentController}interface of the
   * component to which this controller object belongs.
   */
  public ContentController _this_weaveableOptCC;

}
