/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.logger;

import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.julia.InitializationContext;

/**
 * Provides an optional {@link LoggerController }field to a
 * {@link org.objectweb.fractal.julia.Controller}.
 */
public abstract class UseLoggerControllerMixin
{

  // -------------------------------------------------------------------------
  // Private constructor
  // -------------------------------------------------------------------------

  private UseLoggerControllerMixin()
  {
  }

  // -------------------------------------------------------------------------
  // Fields and methods added and overriden by the mixin class
  // -------------------------------------------------------------------------

  /**
   * The {@link LoggerController}interface of the component to which this
   * controller object belongs.
   */
  public LoggerController weaveableOptLogC;

  /**
   * Initializes the fields of this mixin and then calls the overriden method.
   * 
   * @see org.objectweb.fractal.julia.Controller#initFcController(InitializationContext)
   */
  public void initFcController(final InitializationContext ic)
      throws InstantiationException
  {
    try
    {
      weaveableOptLogC = (LoggerController) ic
          .getInterface("logger-controller");
    }
    catch (InstantiationException e)
    {
      weaveableOptLogC = null;
    }
    _super_initFcController(ic);
  }

  // -------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // -------------------------------------------------------------------------

  /**
   * The
   * {@link org.objectweb.fractal.julia.Controller#initFcController(InitializationContext)}
   * method overriden by this mixin.
   * 
   * @see org.objectweb.fractal.julia.Controller#initFcController(InitializationContext)
   */
  public abstract void _super_initFcController(InitializationContext i)
      throws InstantiationException;

}