/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message;

import org.objectweb.dream.pool.ObjectPool;
import org.objectweb.dream.pool.Recyclable;
import org.objectweb.dream.pool.RecyclableFactory;

class BasicChunkFactoryPoolingImpl<T extends AbstractChunk<T>>
    extends
      ChunkFactoryReference<T> implements RecyclableFactory
{
  private final Class<T>   chunkClass;
  private final T          chunkInstance;
  private final ObjectPool chunkPool;

  BasicChunkFactoryPoolingImpl(final Class<T> chunkClass,
      final MessageManager owner, final ObjectPool chunkPool) throws Exception
  {
    super(owner);
    this.chunkClass = chunkClass;
    this.chunkPool = chunkPool;
    chunkInstance = chunkClass.newInstance();
  }

  @SuppressWarnings("unchecked")
  protected T createChunk()
  {
    return (T) chunkPool.newInstance();
  }

  protected void recycleChunk(final T chunk)
  {
    chunk.recycle();
    chunkPool.recycleInstance(chunk);
  }

  final Class<T> getChunkClass()
  {
    return chunkClass;
  }

  public Recyclable newInstance()
  {
    AbstractChunk<T> newChunk = chunkInstance.newChunk();
    newChunk.initAbstractChunk(this, getOwner());
    return newChunk;
  }
}
